﻿using _123.Data.LocalData.Entity;
using _123.Data.ReportsHistory;
using _123.Data.Repository;
using _123.Domain.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace _123.Domain.UseCase
{
    public class PresenceUseCase
    {
        private readonly SQLPresenceRepositoryImpl _presenceRepositoryImpl;
        private readonly SQLGroupRepositoryImpl _groupRepositoryImpl;
        private readonly SQLUserRepositoryImpl _userRepositoryImpl;
        public List<Presence> GetPresenceByGroup(int groupId)
        {
            var users = _userRepositoryImpl.GetAllUsers.Where(x => x.GroupID == groupId).ToList();

            var presenceByGroup = _presenceRepositoryImpl.GetPresenceByGroup(groupId)
            .Where(x => users.Any(user => user.UserGuid == x.UserGuid))
            .Select(presence => new Presence
            {
                User = new User
                {
                    UserGuid = presence.UserGuid,
                    UserGroup = new Group
                    {
                        ID = groupId,
                        Name = _groupRepositoryImpl.GetAllGroups().First(group => group.ID == groupId).Name
                    },
                    UserFIO = users.First(user => user.UserGuid == presence.UserGuid).UserFIO,
                },
                ClassNum = presence.LessonNumber,
                Date = presence.Date,
                IsAttedance = presence.IsAttedance,
            }).ToList();
            return presenceByGroup;
        }
        public List<Presence> GetPresenceByGroupAndDate(int groupId, DateOnly date)
        {
            var users = _userRepositoryImpl.GetAllUsers.Where(x => x.GroupID == groupId).ToList();

            var presenceByGroup = _presenceRepositoryImpl.GetPresenceByGroup(groupId)
            .Where(x => users.Any(user => user.UserGuid == x.UserGuid && x.Date == date))
            .Select(presence => new Presence
            {
                User = new User
                {
                    UserGuid = presence.UserGuid,
                    UserGroup = new Group
                    {
                        ID = groupId,
                        Name = _groupRepositoryImpl.GetAllGroups().First(group => group.ID == groupId).Name
                    },
                    UserFIO = users.First(user => user.UserGuid == presence.UserGuid).UserFIO,
                },
                ClassNum = presence.LessonNumber,
                Date = presence.Date,
                IsAttedance = presence.IsAttedance,
            }).ToList();
            return presenceByGroup;
        }
        public bool UnCheckAttendance(int firstClass, int lastClass, DateOnly date, Guid userGuid)
        {
            return _presenceRepositoryImpl.UnCheckAttendence(firstClass, lastClass, date, userGuid);
        }
        
    }
}
