﻿using _123.Data.LocalData.Entity;
using _123.Data.RemoteData.RemoteDatabase.DAO;
using _123.Data.ReportsHistory;
using _123.Data.Repository;
using _123.Domain.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace _123.Domain.UseCase
{
    public class UserUseCase
    {
        private readonly SQLUserRepositoryImpl _repositoryUserImpl;
        private readonly SQLGroupRepositoryImpl _repositoryGroupImpl;

        public UserUseCase(SQLUserRepositoryImpl repositoryImpl, SQLGroupRepositoryImpl repositoryGroupImpl)
        {
            _repositoryUserImpl = repositoryImpl;
            _repositoryGroupImpl = repositoryGroupImpl;
        }

        public List<Group> GetAllGroups() => _repositoryGroupImpl.GetAllGroups()
              .Select(it => new Group { ID = it.ID, Name = it.Name }).ToList();

        public List<User> GetAllUsers() => _repositoryUserImpl.GetAllUser()
              .Join(_repositoryGroupImpl.GetAllGroups(),
              user => user.GroupID,
              group => group.ID,
              (user, group) =>
              new User
              {
                  UserFIO = user.UserFIO,
                  UserGuid = user.UserID,
                  UserGroup = new Group { ID = group.ID, Name = group.Name }
              }
            ).ToList();

        public bool RemoveUserByGuid(int userId)
        {
            return _repositoryUserImpl.RemoveUserByGuid(userId);
        }

        public User UpdateUser(User user)
        {
            UserDao userDao = new UserDao { 
                UserFIO = user.UserFIO, 
                GroupID = user.UserGroup.ID, 
                UserID = user.UserGuid 
            };
            UserDao? result = _repositoryUserImpl.UpdateUser(userDao);
            if (result == null) throw new Exception("Не удалось обновить пользователя");
            Group? group = GetAllGroups().FirstOrDefault(it => it.ID == result.GroupID);
            if (group == null) throw new Exception("Группа не найдена");
            return new User { UserFIO = user.UserFIO, UserGuid = user.UserGuid, UserGroup = group };
        }

        public UserDao FindUserByGuid(int userId)
        {
            return _repositoryUserImpl.FindUserByGuid(userId);
        }

        public UserDao UpdateUserByGuid(int userId, string name, string groupId)
        {
            UserDao? result = _repositoryUserImpl.UpdateUserByGuid(userId);
            if (result == null) throw new Exception("Пользователь не найден");
            Group? group = GetAllGroups().FirstOrDefault(it => it.ID == int.Parse(groupId));
            if (group == null) throw new Exception("Группа не найдена");
            return new UserDao { UserFIO = name, GroupID = int.Parse(groupId), UserID = userId };
        }
    }
}
